#ifndef RT_GFX_H
#define RT_GFX_H

/* graphics system. this is the interface of the rendering code.
 *
 * we need (at least) three different renderers:
 * - world cell renderer (for world & dungeon environments)
 * - character renderer (for animated models)
 * - object renderer (for static models)
 */

#include <stdbool.h>
#include <stdint.h>

#include "runtime.h"

#ifdef __cplusplus
extern "C" {
#endif

/* In renderer_api.h -> Not necessary for almost all gfx usage */
typedef struct rt_renderer_init_info_s rt_renderer_init_info;

RT_DLLEXPORT void rtRegisterRendererCVars(void);

RT_DLLEXPORT bool rtInitGFX(rt_renderer_init_info *renderer_info);

RT_DLLEXPORT void rtShutdownGFX(void);

/* Handles backend objects */

#define RT_GFX_HANDLE_MAX_VERSION 255

typedef struct {
    uint32_t version : 8;
    uint32_t index : 24;
} rt_pipeline_handle;

/* Attributes are used to bind buffers (or textures) to symbolic values.
 * For example, an attribute might be bound to "CELL_GRID", which would be
 * replaced with the (at the time of the invoke) grid buffer of the current
 * world cell.
 */

typedef enum {
    RT_ATTRIBUTE_VALUE_UNDEFINED,

    RT_ATTRIBUTE_VALUE_MATERIAL_ALBEDO,
    RT_ATTRIBUTE_VALUE_MATERIAL_NORMAL,
} rt_attribute_value;

typedef struct {
    uint32_t index;
    rt_attribute_value value;
} rt_attribute_binding;

#ifdef __cplusplus
}
#endif

#endif
