#include "runtime.h"
#include "config.h"
#include "aio.h"
#include "file_tab.h"
#include "buffer_manager.h"

extern rt_cvar rt_Renderer;
extern rt_cvar rt_Fullscreen;
extern rt_cvar rt_WindowWidth;
extern rt_cvar rt_WindowHeight;
extern rt_cvar rt_BufferMemoryBudget;
extern rt_cvar rt_FileTabCapacity;
extern rt_cvar rt_MaxConcurrentAsyncIO;
extern rt_cvar rt_ResourceDirectory;
extern rt_cvar rt_ResourceCacheSize;
extern rt_cvar rt_ResourceNamespaceSize;

#ifdef RT_BUILD_ASSET_COMPILER
extern rt_cvar rt_AssetDirectory;
#endif

void RegisterRuntimeCVars(void) {
    rtRegisterCVAR(&rt_Renderer);
    rtRegisterCVAR(&rt_Fullscreen);
    rtRegisterCVAR(&rt_WindowWidth);
    rtRegisterCVAR(&rt_WindowHeight);
    rtRegisterCVAR(&rt_BufferMemoryBudget);
    rtRegisterCVAR(&rt_FileTabCapacity);
    rtRegisterCVAR(&rt_MaxConcurrentAsyncIO);
    rtRegisterCVAR(&rt_ResourceDirectory);
    rtRegisterCVAR(&rt_ResourceCacheSize);
    rtRegisterCVAR(&rt_ResourceNamespaceSize);
#ifdef RT_BUILD_ASSET_COMPILER
    rtRegisterCVAR(&rt_AssetDirectory);
#endif
}

extern void SetMainThreadId(void);

extern rt_result InitBufferManager(void);
extern void ShutdownBufferManager(void);
extern rt_result InitFileTab(void);
extern void ShutdownFileTab(void);
extern rt_result InitAIO(void);
extern void ShutdownAIO(void);
extern rt_result InitResourceManager(void);
extern void ShutdownResourceManager(void);

#ifdef RT_BUILD_ASSET_COMPILER
extern rt_result InitAssetCompiler(void);
extern void ShutdownAssetCompiler(void);
#endif

RT_DLLEXPORT rt_result rtInitRuntime(void) {
    SetMainThreadId();
    RegisterRuntimeCVars();

    rt_result res;
    if ((res = InitBufferManager()) != RT_SUCCESS) {
        rtReportError("BUFFERMGR", "Init failed.");
        return res;
    }

    if ((res = InitFileTab()) != RT_SUCCESS) {
        rtReportError("FTAB", "Init failed.");
        return res;
    }

    if ((res = InitAIO()) != RT_SUCCESS) {
        rtReportError("AIO", "Init failed.");
        return res;
    }

    if ((res = InitResourceManager()) != RT_SUCCESS) {
        rtReportError("RESMGR", "Init failed.");
        return res;
    }

#ifdef RT_BUILD_ASSET_COMPILER
    if ((res = InitAssetCompiler()) != RT_SUCCESS) {
        rtReportError("AC", "Init failed.");
        return res;
    }
#endif

    return RT_SUCCESS;
}

RT_DLLEXPORT void rtShutdownRuntime(void) {
#ifdef RT_BUILD_ASSET_COMPILER
    ShutdownAssetCompiler();
#endif
    ShutdownResourceManager();
    ShutdownAIO();
    ShutdownFileTab();
    ShutdownBufferManager();
}