#include "mem_arena.h"

#include <stdlib.h>

#define ALIGNMENT 0xf
#define ALIGN(n)  (((n) + ALIGNMENT) & ~ALIGNMENT)

extern void *memset(void *, int, size_t);

RT_DLLEXPORT rt_create_arena_result rtCreateArena(void *memory, size_t size) {
    rt_arena arena = {.needs_free = 0};
    if (!memory) {
        size   = ALIGN(size);
        memory = malloc(size);
        if (!memory) {
            return (rt_create_arena_result){
                .ok = 0,
            };
        }
        arena.needs_free = 1;
    }
    arena.base = memory;
    arena.size = size;
    arena.at   = 0;
    return (rt_create_arena_result){
        .arena = arena,
        .ok    = 1,
    };
}

RT_DLLEXPORT void *rtArenaPush(rt_arena *arena, size_t n) {
    n = ALIGN(n);
    if (arena->at + n > arena->size)
        return NULL;
    void *p = (char *)arena->base + arena->at;
    arena->at += n;
    return p;
}

RT_DLLEXPORT void *rtArenaPushZero(rt_arena *arena, size_t n) {
    void *p = rtArenaPush(arena, n);
    if (p)
        memset(p, 0, ALIGN(n));
    return p;
}

RT_DLLEXPORT void rtArenaPop(rt_arena *arena, size_t n) {
    n = ALIGN(n);
    if (arena->at < n)
        return;
    arena->at -= n;
}

RT_DLLEXPORT void rtReleaseArena(rt_arena *arena) {
    if (arena->needs_free)
        free(arena->base);
    arena->base       = NULL;
    arena->at         = 0;
    arena->size       = 0;
    arena->needs_free = 0;
}
