#ifndef RT_ASSETC_PROCESSING_H
#define RT_ASSETC_PROCESSING_H

#include "runtime/assets.h"
#include "runtime/file_tab.h"

enum {
    RT_PROCESSING_FAILED = RT_CUSTOM_ERROR_START,
    /* Used if the processing depends on other files beeing processed first. */
    RT_PROCESSING_TRY_AGAIN,
};

typedef struct {
    rt_file_id output_file;

} rt_asset_options;

typedef struct {
    rt_uid asset_uid;

    /* Allocated via the rtAllocBuffer API */
    void *data;
    size_t size;
} rt_processor_output;

typedef rt_result rt_processor_fn(rt_file_id file,
                                  void *buffer,
                                  size_t size,
                                  uint32_t flags,
                                  rt_processor_output *output);

rt_result rtAddAssetProcessor(const char *file_extension, rt_processor_fn fn);

/* Flags are file type specific */
rt_result rtAddFileToProcessingQueue(rt_file_id file, uint32_t flags);

rt_result rtStartProcessing(void);
void rtStopProcessing(void);

rt_uid rtCalculateUID(const char *name);
rt_result rtWriteUIDTab(void);
rt_result rtAddUIDTabEntry(rt_file_id package_fid, rt_uid uid, uint64_t offset, uint64_t size);

void rtWaitUntilProcessingIsFinished(void);

#endif
