#include "utils.h"

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#endif

size_t rtGetFileSize(const char *path) {
#ifdef _WIN32
    WCHAR wpath[MAX_PATH];
    MultiByteToWideChar(CP_UTF8, MB_PRECOMPOSED, path, -1, wpath, MAX_PATH);
    WIN32_FILE_ATTRIBUTE_DATA attribs;
    if (!GetFileAttributesExW(wpath, GetFileExInfoStandard, &attribs))
        return 0;
    return (size_t)attribs.nFileSizeHigh << 32 | (size_t)attribs.nFileSizeLow;
#endif
}

void rtSetWorkingDirectory(const char *path) {
#ifdef _WIN32
    WCHAR wpath[MAX_PATH];
    MultiByteToWideChar(CP_UTF8, MB_PRECOMPOSED, path, -1, wpath, MAX_PATH);
    SetCurrentDirectoryW(wpath);
#endif
}

uint64_t rtGetCurrentTimestamp(void) {
#ifdef _WIN32
    FILETIME ft;
    GetSystemTimeAsFileTime(&ft);
    uint64_t ts = ft.dwLowDateTime;
    ts |= (uint64_t)ft.dwHighDateTime << 32;
    return ts;
#endif
}

uint64_t rtGetFileModificationTimestamp(rt_file_id fid) {
#ifdef _WIN32
    const char *path = rtGetFilePath(fid);
    if (!path) {
        rtLog("ASSETC", "Tried to get modification timestamp of unknown file.");
        return 0;
    }
    WCHAR wpath[MAX_PATH];
    MultiByteToWideChar(CP_UTF8, MB_PRECOMPOSED, path, -1, wpath, MAX_PATH);
    WIN32_FILE_ATTRIBUTE_DATA attribs;
    if (!GetFileAttributesExW(wpath, GetFileExInfoStandard, &attribs))
        return 0;
    uint64_t ts = attribs.ftLastWriteTime.dwLowDateTime;
    ts |= (uint64_t)attribs.ftLastWriteTime.dwHighDateTime << 32;
    return ts;
#endif
}

rt_result rtIterateDirectory(const char *path, void *user, rt_iterate_directory_cb_func iterate_cb) {
#ifdef _WIN32
    rt_result res;
    WCHAR wpath[MAX_PATH];
    char wildcard_path[MAX_PATH];
    strncpy(wildcard_path, path, MAX_PATH);
    strncat(wildcard_path, "\\*", MAX_PATH - strlen(path));

    res = rtUTF8ToWStr(wildcard_path, wpath, MAX_PATH);
    if (res != RT_SUCCESS)
        return res;

    WIN32_FIND_DATAW find;
    HANDLE h = FindFirstFileW(wpath, &find);
    if (h == INVALID_HANDLE_VALUE)
        return RT_UNKNOWN_ERROR;
    do {
        char utf8_file[MAX_PATH];
        res = rtWStrToUTF8(find.cFileName, utf8_file, MAX_PATH);
        if (res != RT_SUCCESS)
            break;
        rtIterateDirElementType type;
        if (find.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
            type = RT_DIR_ELEMENT_TYPE_DIRECTORY;
        else
            type = RT_DIR_ELEMENT_TYPE_FILE;
        res = iterate_cb(utf8_file, type, user);
        if (res != RT_SUCCESS)
            break;
    } while (FindNextFileW(h, &find) != 0);

    FindClose(h);
    return res;
#endif
}

rt_result rtCreateDirectory(const char *path) {
#ifdef _WIN32
    WCHAR wpath[MAX_PATH];
    rt_result res = rtUTF8ToWStr(path, wpath, MAX_PATH);
    if (res != RT_SUCCESS)
        return res;
    if (!CreateDirectoryW(wpath, NULL))
        res = RT_UNKNOWN_ERROR;
    return res;
#endif
}